@@ -8,7 +8,7 @@ import sys
 import time
 from collections import OrderedDict
 from enum import Enum
-from typing import IO, Any, AnyStr, Dict, List, NamedTuple, Optional
+from typing import IO, Any, AnyStr, Dict, List, NamedTuple, Optional, TextIO, Tuple
 
 import inject
 import requests
@@ -50,13 +50,13 @@ class CachedResults(object):
         self._max_cache_size = max_cache_size
         self._cached_results = OrderedDict(initial_cache or {})
 
-    def insert(self, key: str, value: str) -> Dict[str, str] or None:
+    def insert(self, key: str, value: str) -> Optional[Dict[str, str]]:
         """
         Insert new data into cache.
 
-        :param key: key string
-        :param value: value string
-        :return: inserted data as dict or None (if not possible to insert)
+        :param key: Key string.
+        :param value: Value string.
+        :return: Inserted data as dict or None (if not possible to insert).
         """
         if self._max_cache_size <= 0:
             # we can't insert into 0-length dict
@@ -77,12 +77,12 @@ class CachedResults(object):
 
         return dict(build_id=value)
 
-    def get(self, key: str) -> str or None:
+    def get(self, key: str) -> Optional[str]:
         """
         Try to get object by key.
 
-        :param key: key string
-        :return: value for key
+        :param key: Key string.
+        :return: Value for key.
         """
         if self._max_cache_size <= 0:
             return None
@@ -111,23 +111,31 @@ class PathResolver(object):
 
     def __init__(
         self,
-        host: str = None,
+        host: Optional[str] = None,
         cache_size: int = 0,
-        cache_dir: str = None,
-        client_credentials_scope: str = None,
-        client_credentials_user_name: str = None,
-        client_id: str = None,
-        client_secret: str = None,
-        redirect_port: int = None,
-        scope: str = None,
-        auth_domain: str = None,
+        cache_dir: Optional[str] = None,
+        client_credentials_scope: Optional[str] = None,
+        client_credentials_user_name: Optional[str] = None,
+        client_id: Optional[str] = None,
+        client_secret: Optional[str] = None,
+        redirect_port: Optional[int] = None,
+        scope: Optional[str] = None,
+        auth_domain: Optional[str] = None,
     ):
         """
         Initialize instance.
 
-        :param host: URL of host - web service
-        :param cache_size: size of cache. We try to cache recent results and use them instead of asking from server.
-        Use 0 (by default) to disable caching
+        :param host: URL of web service running the API to get debug symbol URL.
+        :param cache_size: Size of cache. We try to cache recent results and use them instead of asking from server.
+            Use 0 (by default) to disable caching.
+        :param cache_dir: Full path to a directory to store cache/files.
+        :param client_credentials_scope: Client credentials scope.
+        :param client_credentials_user_name: Client credentials username.
+        :param client_id: Client id for Okta Oauth.
+        :param client_secret: Secret key for Okta Oauth.
+        :param redirect_port: Redirect port.
+        :param scope: Auth scope.
+        :param auth_domain: Auth domain.
         """
         self.host = host or self.default_host
         self._cached_results = CachedResults(max_cache_size=cache_size)
@@ -160,9 +168,8 @@ class PathResolver(object):
 
         self.authenticate()
 
-    def authenticate(self):
+    def authenticate(self) -> None:
         """Login & get credentials for further requests to web service."""
-
         # try to read from file
         if os.path.exists(self.default_creds_file_path):
             with open(self.default_creds_file_path) as cfile:
@@ -201,27 +208,28 @@ class PathResolver(object):
         Sometimes the given path may not be valid: e.g: path for a non-existing file.
 
         If we need to do extra checks on path, we'll do all of them here.
-        :param path: path string
-        :return: bool indicating the validation status
+
+        :param path: Path string.
+        :return: Bool indicating the validation status.
         """
         return os.path.exists(path)
 
-    def get_from_cache(self, key: str) -> str or None:
+    def get_from_cache(self, key: str) -> Optional[str]:
         """
         Try to get value from cache.
 
-        :param key: key string
-        :return: value or None (if doesn't exist)
+        :param key: Key string.
+        :return: Value or None (if doesn't exist).
         """
         return self._cached_results.get(key)
 
-    def add_to_cache(self, key: str, value: str) -> Dict[str, str]:
+    def add_to_cache(self, key: str, value: str) -> Optional[Dict[str, str]]:
         """
         Add new value to cache.
 
-        :param key: key string
-        :param value: value string
-        :return: added data as dict
+        :param key: Key string.
+        :param value: Value string.
+        :return: Added data as dict.
         """
         return self._cached_results.insert(key, value)
 
@@ -230,8 +238,8 @@ class PathResolver(object):
         """
         Convert URL to local filename.
 
-        :param url: download URL
-        :return: full name for local file
+        :param url: Download URL.
+        :return: Full name for local file.
         """
         return url.split("/")[-1]
 
@@ -240,8 +248,8 @@ class PathResolver(object):
         """
         Use to utar/unzip files.
 
-        :param path: full path of file
-        :return: full path of 'bin' directory of unpacked file
+        :param path: Full path of file.
+        :return: Full path of 'bin' directory of unpacked file.
         """
         out_dir = path.replace(".tgz", "", 1)
         if not os.path.exists(out_dir):
@@ -253,12 +261,12 @@ class PathResolver(object):
 
         return out_dir
 
-    def download(self, url: str) -> (str, bool):
+    def download(self, url: str) -> Tuple[str, bool]:
         """
         Use to download file from URL.
 
-        :param url: URL string
-        :return: full path of downloaded file in local filesystem, bool indicating if file is already downloaded or not
+        :param url: URL string.
+        :return: Full path of downloaded file in local filesystem, bool indicating if file is already downloaded or not.
         """
         exists_locally = False
         filename = self.url_to_filename(url)
@@ -279,7 +287,7 @@ class PathResolver(object):
         To get path for given buildId.
 
         :param soinfo: Information about process that printed backtrace.
-        :return: Path as string or None (if path not found)
+        :return: Path as string or None (if path not found).
         """
         build_id = soinfo.build_id.lower() if soinfo.build_id is not None else ""
         version = soinfo.mongodb_version
@@ -373,7 +381,7 @@ class SymbolizerParameters(NamedTuple):
     * client_id: Client id for Okta Oauth.
     """
 
-    symbolizer_path: str
+    symbolizer_path: Optional[str]
     dsym_hint: List[str]
 
     input_format: InputFormat
@@ -381,12 +389,12 @@ class SymbolizerParameters(NamedTuple):
 
     live: bool
 
-    host: str
-    cache_dir: str
+    host: Optional[str]
+    cache_dir: Optional[str]
     total_seconds_for_retries: int
 
-    client_secret: str
-    client_id: str
+    client_secret: Optional[str]
+    client_id: Optional[str]
 
 
 class SymbolizerOrchestrator:
@@ -416,12 +424,11 @@ class SymbolizerOrchestrator:
 
         :param params: Symbolizer parameters.
         """
-
         # Ignore Ctrl-C. When the process feeding the pipe exits, `stdin` will be closed.
         signal.signal(signal.SIGINT, signal.SIG_IGN)
 
         print("Live mode activated, waiting for input...")
-        global_base_addr_map = {}
+        global_base_addr_map: Dict[str, SoInfo] = {}
         while True:
             backtrace_indicator = '{"backtrace":'
             line = sys.stdin.readline()
@@ -457,7 +464,7 @@ class SymbolizerOrchestrator:
         Execute symbolizer.
 
         :param params: Symbolizer parameters.
-        :return: Succeeded or not.
+        :return: Whether succeeded or not.
         """
         if params.live:
             print("Entering live mode")
@@ -476,15 +483,12 @@ class SymbolizerOrchestrator:
             print("Could not find json backtrace object in input", file=sys.stderr)
             return False
 
-        output_fn = None
-        if params.output_format == OutputFormat.JSON:
-            output_fn = json.dump
-        if params.output_format == OutputFormat.CLASSIC:
-            output_fn = self.classic_output
-
         for trace_doc in trace_docs:
             frames = self.symbolize_frames(trace_doc, params)
-            output_fn(frames, sys.stdout, indent=2)
+            if params.output_format == OutputFormat.JSON:
+                json.dump(frames, sys.stdout, indent=2)
+            elif params.output_format == OutputFormat.CLASSIC:
+                self.classic_output(frames, sys.stdout)
 
         return True
 
@@ -495,7 +499,6 @@ class SymbolizerOrchestrator:
         :param trace_doc: Traceback doc.
         :return: List of traceback frames.
         """
-
         frames = []
         for frame in trace_doc.backtrace:
             if BINARY_LOAD_ADDRESS_KEY not in frame:
@@ -539,7 +542,6 @@ class SymbolizerOrchestrator:
         :param params: Symbolizer parameters.
         :return: List of traceback frames.
         """
-
         frames = self.preprocess_frames_with_retries(
             trace_doc,
             params.input_format,
@@ -571,16 +573,19 @@ class SymbolizerOrchestrator:
             if frame["path"] is None:
                 print("Path not found in frame:", frame)
                 continue
-            symbol_line = "CODE {path:} {addr:}\n".format(**frame)
-            symbolizer_process.stdin.write(symbol_line.encode())
-            symbolizer_process.stdin.flush()
-            frame["symbinfo"] = self.extract_symbols(symbolizer_process.stdout)
-        symbolizer_process.stdin.close()
+            if symbolizer_process.stdin:
+                symbol_line = "CODE {path:} {addr:}\n".format(**frame)
+                symbolizer_process.stdin.write(symbol_line.encode())
+                symbolizer_process.stdin.flush()
+            if symbolizer_process.stdout:
+                frame["symbinfo"] = self.extract_symbols(symbolizer_process.stdout)
+        if symbolizer_process.stdin:
+            symbolizer_process.stdin.close()
         symbolizer_process.wait()
         return frames
 
     @staticmethod
-    def extract_symbols(stdout: Optional[IO[AnyStr]]) -> List[Dict[str, Any]]:
+    def extract_symbols(stdout: IO[AnyStr]) -> List[Dict[str, Any]]:
         """
         Extract symbol information from the output of llvm-symbolizer.
 
@@ -597,10 +602,10 @@ class SymbolizerOrchestrator:
         :param stdout: Output of llvm-symbolizer.
         :return: List of symbol information.
         """
-        result = []
+        result: List[Dict[str, Any]] = []
         step = 0
         while True:
-            line = stdout.readline().decode()
+            line = str(stdout.readline())
             if line == "\n":
                 break
             if step == 0:
@@ -622,7 +627,6 @@ class SymbolizerOrchestrator:
         :param input_format: Input format.
         :return: List of traceback frames.
         """
-
         if input_format == InputFormat.CLASSIC:
             return self.produce_frames(trace_doc)
         elif input_format == InputFormat.THIN:
@@ -631,6 +635,7 @@ class SymbolizerOrchestrator:
                 soinfo = SoInfo(build_id=frame.get(BUILD_ID_KEY))
                 frame["path"] = self.dbg_path_resolver.get_dbg_file(soinfo)
             return frames
+        return []
 
     @staticmethod
     def has_high_not_found_paths_ratio(frames: List[Dict[str, Any]]) -> bool:
@@ -658,19 +663,23 @@ class SymbolizerOrchestrator:
         :param total_seconds_for_retries: Timeout for retries.
         :return: List of traceback frames.
         """
-
         retrying = Retrying(
             retry=retry_if_result(self.has_high_not_found_paths_ratio),
             wait=wait_fixed(60),
             stop=stop_after_delay(total_seconds_for_retries),
-            retry_error_callback=lambda retry_state: retry_state.outcome.result(),
+            retry_error_callback=lambda retry_state: retry_state.outcome.result(),  # type: ignore
         )
 
         return retrying(self.preprocess_frames, trace_doc, input_format)
 
     @staticmethod
-    def classic_output(frames, outfile, **kwargs):
-        """Provide classic output."""
+    def classic_output(frames: List[Dict[str, Any]], outfile: TextIO) -> None:
+        """
+        Provide classic output.
+
+        :param frames: List of traceback frames.
+        :param outfile: Output text file stream.
+        """
         for frame in frames:
             symbinfo = frame.get("symbinfo")
             if symbinfo:
