@@ -3,6 +3,10 @@
 import sys
 
 
-def is_windows():
-    """Return True if Windows."""
+def is_windows() -> bool:
+    """
+    Check whether we are on Windows.
+
+    :return: Return True if Windows
+    """
     return sys.platform.startswith("win32") or sys.platform.startswith("cygwin")
