@@ -1,10 +1,10 @@
 """Helper functions to interact with evergreen."""
 import os
 import pathlib
-from typing import List
+from typing import List, Optional
 
 import structlog
-from evergreen import RetryingEvergreenApi
+from evergreen import EvergreenApi, RetryingEvergreenApi
 
 EVERGREEN_HOST = "https://evergreen.mongodb.com"
 EVERGREEN_CONFIG_LOCATIONS = (
@@ -30,7 +30,7 @@ def _find_evergreen_yaml_candidates() -> List[str]:
     cwd = pathlib.Path(os.getcwd())
     # add every path that is the parent of CWD as well
     for parent in cwd.parents:
-        candidates.append(parent)
+        candidates.append(str(parent))
 
     # Common for local machines
     candidates.append(os.path.expanduser(os.path.join("~", ".evergreen.yml")))
@@ -44,8 +44,13 @@ def _find_evergreen_yaml_candidates() -> List[str]:
     return out
 
 
-def get_evergreen_api(evergreen_config=None):
-    """Return evergreen API."""
+def get_evergreen_api(evergreen_config: Optional[str] = None) -> EvergreenApi:
+    """
+    Return evergreen API.
+
+    :param evergreen_config: Path to Evergreen auth config.
+    :return: Evergreen API client.
+    """
     if evergreen_config:
         possible_configs = [evergreen_config]
     else:
@@ -67,4 +72,7 @@ def get_evergreen_api(evergreen_config=None):
         "Could not connect to Evergreen with any .evergreen.yml files available on this system",
         config_file_candidates=possible_configs,
     )
-    raise last_ex
+    if last_ex is not None:
+        raise last_ex
+
+    return RetryingEvergreenApi.get_api()
