@@ -7,8 +7,12 @@ from typing import Any, Dict, Optional
 import yaml
 
 
-def mkdtemp_in_build_dir():
-    """Use build/ as the temp directory since it's mapped to an EBS volume on Evergreen hosts."""
+def mkdtemp_in_build_dir() -> str:
+    """
+    Use build/ as the temp directory since it's mapped to an EBS volume on Evergreen hosts.
+
+    :return: Temporary directory path.
+    """
     build_tmp_dir = os.path.join("build", "tmp")
     os.makedirs(build_tmp_dir, exist_ok=True)
 
@@ -16,7 +20,12 @@ def mkdtemp_in_build_dir():
 
 
 def build_bin_path(parent: str) -> Optional[str]:
-    """Find the first bin directory recursively in the parent directory."""
+    """
+    Find the first bin directory recursively in the parent directory.
+
+    :param parent: Path to parent directory.
+    :return: Bin directory path.
+    """
     for path, dirs, _ in os.walk(parent):
         if "bin" in dirs:
             return os.path.join(path, "bin")
